@@ -241,6 +241,22 @@ class LofarLBAOuterPrimaryBeam(PrimaryBeam):
         PrimaryBeam.__init__(self, 81.34, 1.1, 'gaussian')
 
 
+class SkaLowPrimaryBeam(PrimaryBeam):
+
+    name = 'ska_low'
+
+    def __init__(self):
+        PrimaryBeam.__init__(self, 38, 1.02, 'gaussian')
+
+
+class NenuFARPrimaryBeam(PrimaryBeam):
+
+    name = 'nenufar'
+
+    def __init__(self):
+        PrimaryBeam.__init__(self, 25, 1.02, 'gaussian')
+
+
 class NoPrimaryBeam(BasePrimaryBeam):
 
     name = 'no_pb'
@@ -426,6 +442,9 @@ class ImageMetaData(object):
         self.wcs = self.wcs.slice(s)
         self.shape = (x_e - x_s, y_e - y_s)
 
+    def average_freqs(self, n_freqs):
+        self.wcs.wcs.cdelt[2] = n_freqs * self.wcs.wcs.cdelt[2]
+
     def items(self, add_origin=False):
         """Returns the meta data as a (key, value) list"""
         header = dict(self.to_header(add_origin=add_origin))
@@ -974,14 +993,13 @@ class CartDataCube(DataCube):
 
         Returns:
             TYPE: Description
-        """
+    """
         ft_cube = []
         freqs = []
         pr = psutil.progress_report(len(files))
         omega_gauss_warning = False
         compat_warning = False
         idx_ft = None
-        idx_ft_full = None
         meta = None
 
         if theta_fov is not None:
@@ -993,13 +1011,22 @@ class CartDataCube(DataCube):
             header = hdu.header
             data = hdu.data.astype(np.complex128)
 
-            freq = header['CRVAL3']
             res = abs(np.radians(header['CDELT1']))
 
-            lamb = const.c.value / freq
+            freq_start = header['CRVAL3']
+            df = header['CDELT3']
+            nf = header['NAXIS3']
+            freq_end = freq_start + (nf - 1) * df
+            fits_freqs = np.linspace(freq_start, freq_end, nf)
+
+            lamb = const.c.value / fits_freqs
 
             cart_map = np.squeeze(data)
-            nx, ny = cart_map.shape
+
+            if cart_map.ndim == 2:
+                cart_map = cart_map[None, :, :]
+
+            _, nx, ny = cart_map.shape
 
             if convert_jy2k:
                 if 'WSCNORMF' in header and imager_scale_factor is None:
@@ -1028,18 +1055,18 @@ class CartDataCube(DataCube):
 
                 jy2k = ((1e-26 * lamb ** 2) / (2 * const.k_B.value))
                 jypsf2K = jy2k / omega
-                cart_map = cart_map * jypsf2K
+                cart_map = cart_map * jypsf2K[:, None, None]
 
             if theta_fov is not None:
                 n = theta_fov / res
 
                 i = int((nx - n) / 2.)
                 if i > 0:
-                    cart_map = cart_map[i:-i, i:-i]
+                    cart_map = cart_map[:, i:-i, i:-i]
 
             if window_function is not None:
-                mask = window_function.generate_window(cart_map.shape[0])
-                cart_map = cart_map * mask
+                mask = window_function.generate_window(cart_map.shape[1])
+                cart_map = cart_map * mask[None, :, :]
 
             if meta is None:
                 meta = ImageMetaData.from_header(header, (nx, ny))
@@ -1051,16 +1078,16 @@ class CartDataCube(DataCube):
                     window_function.to_meta(meta)
 
             if idx_ft is None:
-                uu, vv, idx_ft = psutil.get_ungrid_vis_idx(cart_map.shape, res, umin, umax)
+                uu, vv, idx_ft = psutil.get_ungrid_vis_idx(cart_map.shape[1:], res, umin, umax)
 
-            ft = psutil.img_to_vis(cart_map)
+            ft = psutil.img_to_vis(cart_map, axes=(1, 2))
 
             uu = uu.flatten()
             vv = vv.flatten()
-            ft = ft[idx_ft].flatten()
 
-            ft_cube.append(ft)
-            freqs.append(freq)
+            for freq, ft_slice in zip(fits_freqs, ft):
+                ft_cube.append(ft_slice[idx_ft])
+                freqs.append(freq)
 
         if int_time is not None and 'PEINTTIM' not in meta:
             meta.set('PEINTTIM', int_time)
@@ -1358,20 +1385,22 @@ class CartDataCube(DataCube):
             print('SB with NaN:', self.freqs[idx_nan])
             self.filter_outliers(idx_nan)
 
-    def filter_min_weight(self, min_weight, replace=False):
+    def filter_min_weight(self, min_weight, replace=False, verbose=True):
         if self.weights is not None and min_weight > 0:
             if replace:
                 idx_uv = self.weights.get() >= min_weight
                 n_filt = np.sum(~idx_uv)
                 n_tot = float(len(self.uu) * len(self.freqs))
-                print('Filtering %s visibilities (%.2f %%)' % (n_filt, n_filt // n_tot * 100))
+                if verbose:
+                    print('Filtering %s visibilities (%.2f %%)' % (n_filt, n_filt // n_tot * 100))
                 self.data[~idx_uv] = 0
                 self.weights.data[~idx_uv] = 0
             else:
-                idx_uv = self.weights.get().min(axis=0) >= min_weight
+                idx_uv = np.median(self.weights.get(), axis=0) >= min_weight
                 n_filt = np.sum(~idx_uv)
                 n_tot = float(len(self.uu))
-                print('Filtering %s modes (%.2f %%)' % (n_filt, n_filt // n_tot * 100))
+                if verbose:
+                    print('Filtering %s modes (%.2f %%)' % (n_filt, n_filt // n_tot * 100))
                 self.filter_uv_from_index(idx_uv)
 
     def filter_sefd_uv(self, max_sefd, min_sefd=0):
@@ -1391,15 +1420,10 @@ class CartDataCube(DataCube):
         self.vv = self.vv[idx_uv]
         self.ru = self.ru[idx_uv]
 
-    def average_freqs(self, n_freqs=None, freq_bw=None):
+    def average_freqs(self, n_freqs):
         freqs = self.freqs
-        if freq_bw is None:
-            bins = np.arange(len(freqs) // n_freqs)
-            digi = np.repeat(bins, n_freqs)[:len(freqs)]
-        else:
-            df = psutil.robust_freq_width(freqs)
-            digi = (np.cumsum(np.repeat(df, len(freqs))) / freq_bw).astype(int)
-            bins = np.arange(np.max(digi) + 1)
+        bins = np.arange(len(freqs) // n_freqs)
+        digi = np.repeat(bins, n_freqs)[:len(freqs)]
 
         new_freqs = np.array([freqs[digi == k].mean() for k in bins])
 
@@ -1417,7 +1441,11 @@ class CartDataCube(DataCube):
             new_w = np.array([w[digi == k, :].sum(axis=0) for k in bins])
             new_weight_cube = CartWeightCube(new_w, self.weights.uu, self.weights.vv, new_freqs, self.weights.meta)
 
-        return self.new_with_data(new_data, weights=new_weight_cube, freqs=new_freqs)
+        avg_cube = self.new_with_data(new_data, weights=new_weight_cube, freqs=new_freqs)
+        avg_cube.meta = self.meta.copy()
+        avg_cube.meta.average_freqs(n_freqs)
+
+        return avg_cube
 
     def average_same_uv(self):
 
@@ -1456,7 +1484,7 @@ class CartDataCube(DataCube):
                 fmhz = self.freqs[0] + (self.freqs[-1] - self.freqs[0]) / 2.
             elif fmhz == 'first':
                 fmhz = self.freqs[0]
-            elif fmhz == 'first':
+            elif fmhz == 'last':
                 fmhz = self.freqs[-1]
 
             i = np.argmin(abs(self.freqs - fmhz * 1e6))
