@@ -8,6 +8,7 @@ from __future__ import absolute_import
 
 import numpy as np
 
+import matplotlib as mpl
 import matplotlib.pyplot as plt
 from matplotlib.colors import LogNorm
 
@@ -23,6 +24,15 @@ from . import sphcube
 from . import datacube
 
 
+MPL_VERSION = mpl.__version__.split('.')
+
+# compatibility for MPL <= 3.2
+if int(MPL_VERSION[0]) + 0.1 * int(MPL_VERSION[1]) >= 3.3:
+    nonpos_arg = {'nonpositive': 'clip'}
+else:
+    nonpos_arg = {'nonposy': 'clip'}
+
+
 class EorBin(object):
 
     def __init__(self, name, freqs, freqs_fg, M=None):
@@ -285,7 +295,7 @@ class PowerSpectraBuilder(object):
             ps_config = PowerSpectraConfig()
         elif not isinstance(ps_config, PowerSpectraConfig):
             ps_config = PowerSpectraConfig.load(ps_config)
-        
+
         if eor_bin_list is None:
             eor_bin_list = EorBinList()
             eor_bin_list.add_freq('0', 10, 200)
@@ -377,14 +387,16 @@ class BasePowerSpectra(object):
         self.k_per = psutil.l_to_k(self.el, self.z)
         self.k_par = psutil.delay_to_k(self.delay, self.z)
 
-        # if self.config.filter_kpar_min is not None:
-        #     idx = abs(self.k_par) > self.config.filter_kpar_min
-        #     self.k_par = self.k_par[idx]
-        #     self.delay = self.delay[idx]
-
         self.all_k = np.sqrt(self.k_per ** 2 + self.k_par[:, np.newaxis] ** 2)
         self.kmin = self.all_k.min()
 
+        if self.config.filter_wedge_theta > 0:
+            wedge_kpar = psutil.wedge_fct(np.radians(90), self.z, self.k_per)
+            if self.config.filter_kpar_min is not None:
+                wedge_kpar += self.config.filter_kpar_min
+            wedge = np.array([abs(self.k_par) < w_b for w_b in wedge_kpar]).T
+            self.kmin = self.all_k[~wedge].min()
+
         self.ps2d_pos_only = self.config.ps2d_pos_only
 
     def _compute_window(self, freqs, window_fct):
@@ -771,8 +783,9 @@ class PowerSpectraCart(BasePowerSpectra):
 
     def get_variance(self, data_cube, with_cov_err=True, weighted='default'):
         cl = self.get_cl(data_cube, with_cov_err=with_cov_err, weighted=weighted)
-        var = (cl.data * cl.el * (cl.el.max() - cl.el.min())).sum(axis=1) / (2 * np.pi * len(cl.el))
-        var_err = ((cl.err * cl.el * (cl.el.max() - cl.el.min())) ** 2).sum(axis=1) ** 0.5 / (2 * np.pi * len(cl.el))
+        var = np.nansum(cl.data * cl.el * (cl.el.max() - cl.el.min()), axis=1) / (2 * np.pi * len(cl.el))
+        var_err = np.nansum((cl.err * cl.el * (cl.el.max() - cl.el.min())) ** 2,
+                            axis=1) ** 0.5 / (2 * np.pi * len(cl.el))
 
         return Variance(var, var_err, cl.freqs)
 
@@ -816,24 +829,7 @@ class PowerSpectraCart(BasePowerSpectra):
         a = self.get_ps3d(kbins, ft_cube, with_cov_err=with_cov_err, weighted=weighted)
         n = self.get_ps3d(kbins, noise_cube, with_cov_err=with_cov_err, weighted=weighted)
 
-        d = a - n
-
-        ps_err_norm = self.get_ps_err_norm(ft_cube)
-        ps_err_norm_signal = self.get_ps_err_norm(ft_cube, with_pb=True)
-
-        k_per_full = psutil.l_to_k(ft_cube.ru * 2 * np.pi, self.z)
-
-        k_per = np.repeat(k_per_full[np.newaxis, :], len(self.k_par), axis=0)
-        k_par = np.repeat(self.k_par[:, np.newaxis], len(ft_cube.ru), axis=1)
-        ks = np.sqrt(k_per ** 2 + k_par ** 2)
-
-        vis_count = pscart.get_vis_count(ks, kbins)
-        d_err = np.sqrt(2 / vis_count) * d.data
-
-        d.err = np.sqrt(2 * (n.err * ps_err_norm) ** 2 + (d_err * ps_err_norm_signal) ** 2)
-        d.n_eff = n.n_eff
-
-        return d
+        return a - n
 
     def get_cross_ps3d(self, kbins, data_cube1, data_cube2, with_cov_err=True, weighted='default'):
         ''' Return spherically averaged power spectra of data_cube for kbins'''
@@ -851,8 +847,8 @@ class PowerSpectraCart(BasePowerSpectra):
         k_per = np.repeat(k_per_full[np.newaxis, :], len(self.k_par), axis=0)
         k_par = np.repeat(self.k_par[:, np.newaxis], len(data_cube1.ru), axis=1)
 
-        k_mean, dsp, dsp_err, n_eff = pscart.get_3d_cross_power_spectre(
-            dft_cube1, dft_cube2, kbins, k_per, k_par, weight_cube=weight_cube)
+        k_mean, k_std, dsp, dsp_err, n_eff = pscart.get_3d_cross_power_spectre(dft_cube1, dft_cube2, kbins,
+                                                                               k_per, k_par, weight_cube=weight_cube)
 
         dsp = dsp * ps2d_norm
         dsp_err = dsp_err * ps2d_norm
@@ -1071,7 +1067,7 @@ class Variance(PowerSpectraMath):
             e = e * 1e6
 
         ax.errorbar(f * 1e-6, d, nsigma * e, **kargs)
-        ax.set_yscale('log', nonposy='clip')
+        ax.set_yscale('log', **nonpos_arg)
         ax.set_xlabel(r"$\mathrm{Frequency\,[MHz]}$")
 
         if mkelvin:
@@ -1221,7 +1217,7 @@ class SpatialPowerSpectra(PowerSpectraMath):
             ax.errorbar(x, y, nsigma * y_err, **kargs)
         else:
             ax.plot(x, y, **kargs)
-        ax.set_yscale('log', nonposy='clip')
+        ax.set_yscale('log', **nonpos_arg)
 
     def save_to_txt(self, filename):
         k_pers, freqs = np.meshgrid(self.k_per, self.freqs)
@@ -1420,7 +1416,7 @@ class CylindricalPowerSpectra(PowerSpectraMath):
             ax.errorbar(x, y, nsigma * y_err, **kargs)
         else:
             ax.plot(x, y, **kargs)
-        ax.set_yscale('log', nonposy='clip')
+        ax.set_yscale('log', **nonpos_arg)
 
         if delay:
             ax.set_xlabel('Delay (us)')
@@ -1452,7 +1448,7 @@ class CylindricalPowerSpectra(PowerSpectraMath):
             ax.errorbar(self.k_per, y, nsigma * y_err, **kargs)
         else:
             ax.plot(self.k_per, y, **kargs)
-        ax.set_yscale('log', nonposy='clip')
+        ax.set_yscale('log', **nonpos_arg)
 
         ax.set_xlabel(r'$k_{\bot}\,[\mathrm{h\,cMpc^{-1}}]$')
 
@@ -1566,7 +1562,7 @@ class SphericalPowerSpectra(PowerSpectraMath):
         else:
             ax.plot(self.k_mean, d, marker=marker, **kargs)
 
-        ax.set_yscale('log', nonposy='clip')
+        ax.set_yscale('log', **nonpos_arg)
         ax.set_xscale('log')
 
         ax.set_ylabel(r'$\Delta%s (k)\,[\mathrm{%s}]$' % ('^2' * kelvin_square, temp_unit))
