@@ -316,6 +316,13 @@ def wedge_fct(theta, z, k_per):
     return np.sin(theta) * (dm * cosmo.H(z) / (const.c * (1 + z))).decompose().value * k_per
 
 
+def robust_freq_width(freqs):
+    '''Return frequency width, robust to gaps in freqs'''
+    dfs = np.diff(freqs)
+    m, idx, c = np.unique(np.round(dfs * 1e-3) * 1e3, return_counts=True, return_inverse=True)
+    return dfs[np.where(idx == np.argmax(c))].mean()
+
+
 def nudft(x, y, M=None, w=None, dx=None):
     """Non uniform discrete Fourier transform
 
@@ -559,8 +566,12 @@ def sort_by_sb(files):
     return sorted(files, key=key_fct)
 
 
+def get_fits_key(files, key):
+    return np.array([FastHeaderReader(f, keys=[key]).get_value(key) for f in files])
+
+
 def sort_by_fits_key(files, key):
-    values = np.array([FastHeaderReader(f, keys=[key]).get_value(key) for f in files])
+    values = get_fits_key(files, key)
     files_values = sorted(list(zip(files, values)), key=lambda k: k[1])
     return list(zip(*files_values))[0]
 
