@@ -5,19 +5,21 @@ from __future__ import absolute_import
 
 import os
 import time
+import itertools
+
 import click
 
 import numpy as np
 
-import astropy.units as u
-
 import matplotlib as mpl
 mpl.use('Agg')
 
 import matplotlib.pyplot as plt
 
+import astropy.constants as const
+
+# All ps_eor modules are lazy loaded
 from ps_eor import __version__
-from ps_eor import datacube, psutil, pspec, flagger, sphcube, fgfit, fitutil
 
 
 mpl.rcParams['image.cmap'] = 'Spectral_r'
@@ -28,6 +30,10 @@ mpl.rcParams['axes.grid'] = True
 t_file = click.Path(exists=True, dir_okay=False)
 t_dir = click.Path(exists=True, file_okay=False)
 
+INSTRU_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'instru')
+
+MIN_EOR_BW_MHZ = 2
+
 
 @click.group()
 @click.version_option(__version__)
@@ -39,7 +45,7 @@ def main():
 @click.argument('img_list', type=t_file)
 @click.argument('psf_list', type=t_file)
 @click.option('--output_name', '-o', help='Output filename', default='datacube.h5', show_default=True)
-@click.option('--theta_fov', help='Trim image to FoV (in degrees)', default=4, type=float, show_default=True)
+@click.option('--theta_fov', help='Trim image to FoV (in degrees)', default='4', type=str, show_default=True)
 @click.option('--umin', help='Min baseline (in lambda)', default=50, type=float, show_default=True)
 @click.option('--umax', help='Max baseline (in lambda)', default=250, type=float, show_default=True)
 @click.option('--threshold', '-w', help='Filter all visibilities with weights below T times the max weights',
@@ -57,8 +63,10 @@ def gen_vis_cube(img_list, psf_list, output_name, theta_fov, umin, umax, thresho
         \b
         IMG_LIST: Listing of input fits image files
         PSF_LIST: Listing of input fits psf files
-
     '''
+    import astropy.units as u
+    from ps_eor import datacube, psutil
+
     if int_time is not None:
         int_time = u.Quantity(int_time, u.s).to(u.s).value
 
@@ -79,6 +87,13 @@ def gen_vis_cube(img_list, psf_list, output_name, theta_fov, umin, umax, thresho
         win_fct = datacube.WindowFunction(name)
         print('Using mask: %s' % win_fct)
 
+    if theta_fov.endswith('lamb'):
+        mfreq = psutil.get_fits_key(imgs, 'CRVAL3').mean()
+        lamb = 299792458. / mfreq
+        theta_fov = float(theta_fov[:-4]) * lamb
+    else:
+        theta_fov = float(theta_fov)
+
     data_cube = datacube.CartDataCube.load_from_fits_image_and_psf(imgs, psfs, umin, umax,
                                                                    np.radians(theta_fov),
                                                                    int_time=int_time, total_time=total_time,
@@ -98,6 +113,8 @@ def gen_vis_cube(img_list, psf_list, output_name, theta_fov, umin, umax, thresho
 @click.argument('diff')
 def even_odd_to_sum_diff(even, odd, sum, diff):
     ''' Create SUM / DIFF datacubes from EVEN / ODD datacubes '''
+    from ps_eor import datacube
+
     even = datacube.CartDataCube.load(even)
     odd = datacube.CartDataCube.load(odd)
     (0.5 * (even + odd)).save(sum)
@@ -113,6 +130,8 @@ def even_odd_to_sum_diff(even, odd, sum, diff):
               type=click.Path(file_okay=False), show_default=True)
 def diff_cube(cube1, cube2, out_file):
     ''' Compute the difference between CUBE1 and CUBE2 '''
+    from ps_eor import datacube
+
     cube1 = datacube.CartDataCube.load(cube1)
     cube2 = datacube.CartDataCube.load(cube2)
 
@@ -134,7 +153,9 @@ def run_flagger(file_i, file_v, flag_config, output_dir):
         FILE_I: Input Stoke I datacube
         FILE_V: Input Stoke V datacube
         FLAG_CONFIG: Flagger configuration file
-        '''
+    '''
+    from ps_eor import datacube, flagger, psutil
+
     print('Loading cube ...')
     i_cube = datacube.CartDataCube.load(file_i)
     v_cube = datacube.CartDataCube.load(file_v)
@@ -172,6 +193,8 @@ def make_ps(file_i, file_v, file_dt, flag_config, eor_bins_list, ps_conf, output
         FILE_V: Input Stoke V datacube
         FILE_DT: Optional input time-diffence datacube
         '''
+    from ps_eor import datacube, pspec, flagger, psutil
+
     if not os.path.exists(output_dir):
         os.makedirs(output_dir)
 
@@ -206,14 +229,12 @@ def make_ps(file_i, file_v, file_dt, flag_config, eor_bins_list, ps_conf, output
     ps_conf.el = 2 * np.pi * np.arange(i_cube.ru.min(), i_cube.ru.max(), ps_conf.du)
 
     for eor_name in ps_builder.eor_bin_list.get_all_names():
-        if not ps_builder.eor_bin_list.get(eor_name, freqs=i_cube.freqs):
+        eor = ps_builder.eor_bin_list.get(eor_name, freqs=i_cube.freqs)
+
+        if not eor or eor.bw_total < MIN_EOR_BW_MHZ * 1e6:
             continue
 
         ps_gen = ps_builder.get(i_cube, eor_name)
-        eor = ps_gen.eor
-
-        if eor.bw_total < 2e6:
-            continue
 
         plot_out_dir = os.path.join(plots_output_dir, 'eor%s_u%s-%s' %
                                     (eor_name, int(ps_conf.umin), int(ps_conf.umax)))
@@ -288,6 +309,8 @@ def make_ps(file_i, file_v, file_dt, flag_config, eor_bins_list, ps_conf, output
 @click.option('--flagfile', '-f', help='Flag to be applied to visibility cube before transformation', type=t_file)
 def vis_to_sph(vis_cube, sph_cube, umin, umax, nside, flagfile):
     ''' Load visibilities datacubes VIS_CUBE, transform to sph datacube and save to SPH_CUBE '''
+    from ps_eor import datacube, sphcube, flagger
+
     cube = datacube.CartDataCube.load(vis_cube)
 
     if flagfile is not None:
@@ -307,11 +330,13 @@ def vis_to_sph(vis_cube, sph_cube, umin, umax, nside, flagfile):
 
 
 def do_gpr(i_cube, v_cube, gpr_config_i, gpr_config_v, rnd_seed=1):
+    from ps_eor import fgfit
+
     t = time.time()
     np.random.seed(rnd_seed)
 
-    gpr_config_v = fitutil.GprConfig.load(gpr_config_v)
-    gpr_config_i = fitutil.GprConfig.load(gpr_config_i)
+    gpr_config_v = fgfit.fitutil.GprConfig.load(gpr_config_v)
+    gpr_config_i = fgfit.fitutil.GprConfig.load(gpr_config_i)
 
     print('Running GPR for Stokes V ...\n')
 
@@ -396,9 +421,10 @@ def get_ratios(ps_gen, ps2d_ratio):
 @click.option('--ps_conf', '-c', type=t_file, help='Power spectra configuration file')
 @click.option('--output_dir', '-o', help='Output directory', default='.')
 @click.option('--plots_output_dir', '-po', help='Output directory for the figures')
+@click.option('--no_plot', help='Skip plotting', is_flag=True)
 @click.option('--rnd_seed', help='Set a random seed', default=3, type=int)
 def run_gpr(file_i, file_v, gpr_config_i, gpr_config_v, flag_config, eor_bins_list, ps_conf,
-            output_dir, plots_output_dir, rnd_seed):
+            output_dir, plots_output_dir, no_plot, rnd_seed):
     ''' Run GPR & generate power spectra
 
     \b
@@ -407,6 +433,8 @@ def run_gpr(file_i, file_v, gpr_config_i, gpr_config_v, flag_config, eor_bins_li
     GPR_CONFIG_I: GPR configuration parset for Stokes I
     GPR_CONFIG_V: GPR configuration parset for Stokes V
     '''
+    from ps_eor import datacube, pspec, flagger, psutil
+
     if plots_output_dir is None:
         plots_output_dir = output_dir
 
@@ -431,17 +459,19 @@ def run_gpr(file_i, file_v, gpr_config_i, gpr_config_v, flag_config, eor_bins_li
         flagger_runner = flagger.FlaggerRunner.load(flag_config)
         i_cube, v_cube = flagger_runner.run(i_cube, v_cube)
 
+        if not no_plot:
+            fig = flagger_runner.plot()
+            fig.savefig(os.path.join(plots_output_dir, 'flagger_gpr.pdf'))
+
     ps_conf.el = 2 * np.pi * np.arange(i_cube.ru.min(), i_cube.ru.max(), ps_conf.du)
 
     for eor_name in ps_builder.eor_bin_list.get_all_names():
-        if not ps_builder.eor_bin_list.get(eor_name, freqs=i_cube.freqs):
+        eor = ps_builder.eor_bin_list.get(eor_name, freqs=i_cube.freqs)
+
+        if not eor or eor.bw_total < MIN_EOR_BW_MHZ * 1e6:
             continue
 
         ps_gen = ps_builder.get(i_cube, eor_name)
-        eor = ps_gen.eor
-
-        if eor.bw_total < 2e6:
-            continue
 
         out_dir = os.path.join(output_dir, 'eor%s_u%s-%s' % (eor_name, int(ps_conf.umin), int(ps_conf.umax)))
         plot_out_dir = os.path.join(plots_output_dir, 'eor%s_u%s-%s' %
@@ -504,6 +534,9 @@ def run_gpr(file_i, file_v, gpr_config_i, gpr_config_v, flag_config, eor_bins_li
         upper_limit = '2 sigma upper limit: (%.1f)^2 mK^2 at k = %.3f' % (np.nanmin(up), k_up)
         print(upper_limit)
 
+        if no_plot:
+            continue
+
         print('Plotting results ...\n')
 
         # Plotting GPR results
@@ -573,6 +606,8 @@ def combine(file_list, umin, umax, weights_mode, inhomogeneous, pre_flag, post_f
         OBS_ID CUBE1_I CUBE1_V CUBE1_DT
         OBS_ID CUBE2_I CUBE2_V CUBE2_DT
         ... '''
+    from ps_eor import datacube, flagger
+
     file_list = np.loadtxt(file_list, dtype=str)
 
     i_cubes = []
@@ -701,6 +736,8 @@ def combine_sph(file_list, pre_flag, post_flag, output_template, save_intermedia
         OBS_ID CUBE1_I CUBE1_V CUBE1_DT
         OBS_ID CUBE2_I CUBE2_V CUBE2_DT
         ... '''
+    from ps_eor import sphcube, flagger, psutil
+
     file_list = np.loadtxt(file_list, dtype=str)
 
     i_cubes = []
@@ -765,5 +802,392 @@ def combine_sph(file_list, pre_flag, post_flag, output_template, save_intermedia
             c_dt_cube.save(out_dt)
 
 
+class TelescopeSimu(object):
+
+    name = 'none'
+
+    def __init__(self, freqs, dec_deg, hal, har, umin=None, umax=None, timeres=100):
+        self.freqs = freqs
+        self.dec_deg = dec_deg
+        self.hal = hal
+        self.har = har
+        if umin is not None:
+            self.umin = umin
+        if umax is not None:
+            self.umax = umax
+        self.timeres = timeres
+
+    @staticmethod
+    def from_name(instru, freqs, dec_deg, hal, har, umin=None, umax=None, timeres=100):
+        klasses = TelescopeSimu.__subclasses__()
+        [klasses.extend(k.__subclasses__()) for k in klasses[:]]
+
+        for klass in klasses:
+            if hasattr(klass, 'name') and klass.name == instru:
+                return klass(freqs, dec_deg, hal, har, umin, umax, timeres)
+
+        raise ValueError('No instrument with name: %s' % instru)
+
+    def get_stat_pos_file(self):
+        pass
+
+    def get_sefd(self, freq):
+        pass
+
+    def get_primary_beam(self):
+        pass
+
+    def simu_uv(self, include_conj=True):
+        from ps_eor import psutil
+
+        m2a = lambda m: np.squeeze(np.asarray(m))
+
+        lambs = const.c.value / self.freqs
+        umin_meter = (self.umin * lambs).min()
+        umax_meter = (self.umax * lambs).max()
+
+        timev = np.arange(self.hal * 3600, self.har * 3600, self.timeres)
+
+        statpos = np.loadtxt(self.get_stat_pos_file())
+        nstat = statpos.shape[0]
+
+        # All combinations of nant to generate baselines
+        stncom = itertools.combinations(np.arange(1, nstat), 2)
+
+        b1, b2 = zip(*stncom)
+
+        uu = []
+        vv = []
+        ww = []
+
+        print('Simulating UV coverage ...')
+        pr = psutil.progress_report(len(timev))
+        i = 0
+
+        for tt in timev:
+            pr(i)
+            HA = (tt / 3600.) * (15. / 180) * np.pi - (6.8689389 / 180) * np.pi
+            dec = self.dec_deg * (np.pi / 180)
+            RM = np.matrix([[np.sin(HA), np.cos(HA), 0.0],
+                            [-np.sin(dec) * np.cos(HA), np.sin(dec) * np.sin(HA), np.cos(dec)],
+                            [np.cos(dec) * np.cos(HA), - np.cos(dec) * np.sin(HA), np.sin(dec)]])
+            statposuvw = np.dot(RM, statpos.T).T
+            bu = m2a(statposuvw[b1, 0] - statposuvw[b2, 0])
+            bv = m2a(statposuvw[b1, 1] - statposuvw[b2, 1])
+            bw = m2a(statposuvw[b1, 2] - statposuvw[b2, 2])
+
+            ru = np.sqrt(bu ** 2 + bv ** 2)
+            idx = (ru > umin_meter) & (ru < umax_meter)
+
+            uu.extend(bu[idx])
+            vv.extend(bv[idx])
+            ww.extend(bw[idx])
+
+            if include_conj:
+                uu.extend(- bu[idx])
+                vv.extend(- bv[idx])
+                ww.extend(bw[idx])
+
+            i += 1
+
+        return uu, vv, ww
+
+    def simu_gridded_uv(self, uu_meter, vv_meter, fov, oversampling_factor=4):
+        from ps_eor import psutil, datacube
+
+        du = 1 / np.radians(fov)
+        res = 1 / (oversampling_factor * self.umax)
+        n_u = int(np.ceil(1 / (res * du)))
+        shape = (n_u, n_u)
+
+        g_uu, g_vv = psutil.get_uv_grid(shape, res)
+
+        l = [(a - (b - a) / 2., b + (b - a) / 2.) for a, b in psutil.pairwise(g_uu[0])]
+        bins_edges = np.array([k[0] for k in l] + [l[-2][1], l[-1][1]])
+
+        print('Gridding UV coverage ...')
+        weights = []
+        pr = psutil.progress_report(len(self.freqs))
+        for i, lamb in enumerate(const.c.value / self.freqs):
+            pr(i)
+            w, _, _ = np.histogram2d(uu_meter / lamb, vv_meter / lamb, bins=bins_edges)
+            weights.append(w)
+
+        weights = np.array(weights)
+        weights = weights.reshape(len(self.freqs), -1)
+        g_uu = g_uu.flatten()
+        g_vv = g_vv.flatten()
+        ru = np.sqrt(g_uu ** 2 + g_vv ** 2)
+
+        w_min = 0.01 * np.mean(weights[weights > 0])
+
+        idx = (weights.min(axis=0) > w_min) & (ru >= self.umin) & (ru <= self.umax)
+        weights = weights[:, idx]
+        g_uu = g_uu[idx]
+        g_vv = g_vv[idx]
+
+        meta = datacube.ImageMetaData.from_res(res, shape)
+        meta.wcs.wcs.cdelt[2] = psutil.robust_freq_width(self.freqs)
+        meta.set('PEINTTIM', self.timeres)
+        meta.set('PETOTTIM', (self.har - self.hal) * 3600)
+        w_cube = datacube.CartWeightCube(weights, g_uu, g_vv, self.freqs, meta)
+
+        return TelescopGriddedUV(w_cube, self.name, self.get_sefd(self.freqs.mean()))
+
+
+class TelescopGriddedUV(object):
+
+    def __init__(self, weights, name, sefd):
+        from ps_eor import psutil
+
+        self.weights = weights
+        self.name = name
+        self.sefd = sefd
+        self.z = psutil.freq_to_z(self.weights.freqs.mean())
+
+    def save(self, filename):
+        self.weights.meta.set('PESEFD', self.sefd)
+        self.weights.meta.set('PEINSTRU', self.name)
+        self.weights.save(filename)
+
+    @staticmethod
+    def load(filename):
+        from ps_eor import datacube
+
+        weights = datacube.CartWeightCube.load(filename)
+        sefd = weights.meta.get('PESEFD')
+        name = weights.meta.get('PEINSTRU')
+
+        return TelescopGriddedUV(weights, name, sefd)
+
+    def get_ps_gen(self, filter_kpar_min=None, filter_wedge_theta=0):
+        from ps_eor import pspec, datacube
+
+        du = 1 / self.weights.meta.theta_fov
+
+        el = 2 * np.pi * (np.arange(self.weights.ru.min(), self.weights.ru.max(), du))
+
+        ps_conf = pspec.PowerSpectraConfig(el, window_fct='boxcar')
+        ps_conf.filter_kpar_min = filter_kpar_min
+        ps_conf.filter_wedge_theta = filter_wedge_theta
+
+        eor_bin_list = pspec.EorBinList(self.weights.freqs)
+        eor_bin_list.add_freq(1, self.weights.freqs.min() * 1e-6, self.weights.freqs.max() * 1e-6)
+        eor = eor_bin_list.get(1, self.weights.freqs)
+        pb = datacube.PrimaryBeam.from_name(self.name)
+
+        return pspec.PowerSpectraCart(eor, ps_conf, pb)
+
+
+class SkaLow(TelescopeSimu):
+
+    name = 'ska_low'
+    umin = 30
+    umax = 250
+    fov = 4
+
+    def get_stat_pos_file(self):
+        return os.path.join(INSTRU_DIR, 'skalow_enu_statpos.data')
+
+    def get_sefd(self, freq):
+        # Target SEFD at 115 MHz of 2500 Jy
+        return 2500 * (115e6 / freq) ** 0.55
+
+
+class LofarHBA(TelescopeSimu):
+
+    name = 'lofar_hba'
+    umin = 50
+    umax = 250
+    fov = 4
+
+    def get_stat_pos_file(self):
+        return os.path.join(INSTRU_DIR, 'lofar_statpos.data')
+
+    def get_sefd(self, freq):
+        # Typical observed SEFD ~ 140 MHz @ NCP
+        return 4000
+
+
+class NenuFAR(TelescopeSimu):
+
+    name = 'nenufar'
+    umin = 6
+    umax = 60
+    fov = 16
+
+    def get_stat_pos_file(self):
+        return os.path.join(INSTRU_DIR, 'nenufar52_statpos.data')
+
+    def sky_temperature(self, freq, tsys_sky=60):
+        lamb = const.c.value / freq
+        return tsys_sky * lamb ** 2.55
+
+    def inst_temperature(self, freq):
+        """ Instrument temperature at a given frequency ``freq``.
+
+            From: https://github.com/AlanLoh/nenupy/blob/master/nenupy/instru/instru.py
+        """
+        lna_sky = np.array([
+            5.0965, 2.3284, 1.0268, 0.4399, 0.2113, 0.1190, 0.0822, 0.0686,
+            0.0656, 0.0683, 0.0728, 0.0770, 0.0795, 0.0799, 0.0783, 0.0751,
+            0.0710, 0.0667, 0.0629, 0.0610, 0.0614, 0.0630, 0.0651, 0.0672,
+            0.0694, 0.0714, 0.0728, 0.0739, 0.0751, 0.0769, 0.0797, 0.0837,
+            0.0889, 0.0952, 0.1027, 0.1114, 0.1212, 0.1318, 0.1434, 0.1562,
+            0.1700, 0.1841, 0.1971, 0.2072, 0.2135, 0.2168, 0.2175, 0.2159,
+            0.2121, 0.2070, 0.2022, 0.1985, 0.1974, 0.2001, 0.2063, 0.2148,
+            0.2246, 0.2348, 0.2462, 0.2600, 0.2783, 0.3040, 0.3390, 0.3846,
+            0.4425, 0.5167, 0.6183, 0.7689, 1.0086, 1.4042, 2.0732
+        ])
+        freqs = (np.arange(71) + 15) * 1e6
+        return self.sky_temperature(freq) * lna_sky[np.abs(freqs - freq).argmin()]
+
+    def get_sefd(self, freq, tsys_sky=60):
+        d = 5.5
+        lamb = const.c.value / freq
+        tsys = self.sky_temperature(freq, tsys_sky) + self.inst_temperature(freq)
+        a_eff = 19 * np.min([lamb ** 2 / 3., np.ones_like(lamb) * np.pi * d ** 2 / 4.], axis=0)
+
+        return 2 * const.k_B.value / a_eff * 1e26 * tsys
+
+
+@main.command('simu_uv')
+@click.argument('instru', type=click.Choice(['ska_low', 'nenufar', 'lofar_hba'], case_sensitive=False))
+@click.argument('z', type=float)
+@click.option('--out_filename', '-o', help='Base filename of the output file', type=str, default='simu_uv',
+              show_default=True)
+@click.option('--umin', help='Minimum baseline', default=None, type=int)
+@click.option('--umax', help='Maximum baseline', default=None, type=int)
+@click.option('--dec_deg', help='Target declination, in degrees', default=90, type=int, show_default=True)
+@click.option('--fov', help='Field of View side size (in deg)', default=None, type=float)
+@click.option('--bandwidth', help='Bandwidth, in MHz', default=51 * 195.3 * 1e-3, type=float, show_default=True)
+@click.option('--df', help='Spectral resolution, in kHz', default=195.3, type=float, show_default=True)
+@click.option('--total_time', help='Total observing time (in hours)', default=10, type=float, show_default=True)
+@click.option('--int_time', help='Integration time (in second)', default=100, type=float, show_default=True)
+@click.option('--sefd', help='System Equivalent Flux Density (Jy)', default=None, type=float)
+@click.option('--res', help='Image resolution in degrees. If not given a default 4x oversampling will be used',
+              default=None, type=float)
+def simu_uv(instru, z, out_filename, umin, umax, dec_deg, fov, bandwidth, df, total_time, int_time, sefd, res):
+    ''' Compute gridded UV coverage at redshift Z. Possible choices: ska_low, nenufar or lofar_hba.
+    '''
+    from ps_eor import psutil
+
+    assert total_time < 24, 'total_time of one night must be < 24 hours'
+
+    fmhz = psutil.z_to_freq(z) * 1e-6
+    freqs = np.arange(fmhz - bandwidth / 2, fmhz + bandwidth / 2, df * 1e-3) * 1e6
+    telescop = TelescopeSimu.from_name(instru, freqs, dec_deg, 0, total_time, umin, umax, int_time)
+
+    if fov is None:
+        fov = telescop.fov
+    if sefd is None:
+        sefd = telescop.get_sefd(freqs.mean())
+
+    print(f'Computing gridded UV configuration for instrument {telescop.name}')
+    print(f'Simulating {total_time} hour long observation with {int_time} s integration time')
+    print(f'Redshift z = {z:.1f} (mean frequency = {fmhz:.1f} MHz)')
+    print(f'{len(freqs)} frequency channels between {freqs.min() * 1e-6:.1f} and {freqs.max() * 1e-6:.1f} MHz')
+    print(f'Frequency resolution: {df:.1f} kHz')
+    print(f'Baseline range: {telescop.umin} - {telescop.umax} lambda')
+    print(f'FoV: {fov:.1f} degrees')
+    print(f'SEFD: {sefd:.1f} Jy')
+
+    uu_meter, vv_meter, _ = telescop.simu_uv()
+
+    if res is not None:
+        osamp_factor = int(np.round((1 / telescop.umax) / np.radians(6 / 60.)))
+    else:
+        osamp_factor = 4
+
+    uv_simu = telescop.simu_gridded_uv(uu_meter, vv_meter, fov, osamp_factor)
+
+    print(f'Image size: {uv_simu.weights.meta.shape}')
+    print(f'Image resolution: {np.degrees(uv_simu.weights.meta.res) * 60:.2f} arcmin (oversampling: {osamp_factor})')
+
+    name = f'{out_filename}_{telescop.name}_z{z:.1f}_{total_time}h'
+
+    fig, ax = plt.subplots(figsize=(5, 4))
+    uv_simu.weights.plot_uv(uv_lines=[], ax=ax)
+    ax.set_xlabel('U [lambda]')
+    ax.set_ylabel('V [lambda]')
+    ax.set_title('Gridded UV coverage at mid frequency')
+    fig.tight_layout()
+    fig.savefig(f'{name}.pdf')
+
+    print(f'Saving UV coverage to {name}.h5 ...')
+    uv_simu.save(f'{name}.h5')
+
+
+@main.command('simu_noise_img')
+@click.argument('uv_simu_h5', type=t_file)
+@click.argument('total_time_hour', type=int)
+@click.option('--out_filename', '-o', help='Base filename of the output file', type=str, default='simu_noise')
+def simu_noise_img(uv_simu_h5, total_time_hour, out_filename):
+    ''' Compute noise fits cube from a simulated UV coverage UV_SIMU_H5 for TOTAL_TIME_HOUR observation. '''
+
+    from ps_eor import datacube, psutil
+
+    uv_simu = TelescopGriddedUV.load(uv_simu_h5)
+
+    print(f'Simulating thermal noise with SEFD = {uv_simu.sefd} Jy ...')
+    noise_cube = uv_simu.weights.simulate_noise(uv_simu.sefd, total_time_hour * 3600, hermitian=False)
+    noise_img = noise_cube.image()
+    print(f'RMS noise @ mid frequency {noise_img.data[len(noise_img.freqs) // 2].std()} Kelvin')
+
+    name = f'{out_filename}_{uv_simu.name}_z{uv_simu.z:.1f}_{total_time_hour}h.fits'
+    print(f'Saving noise cube to {name} ...')
+    noise_img.save_to_fits(name)
+
+
+@main.command('simu_noise_ps')
+@click.argument('uv_simu_h5', type=t_file)
+@click.argument('total_time_hour', type=float)
+@click.option('--out_filename', '-o', help='Base filename of the output file', type=str, default='simu_noise')
+@click.option('--n_simu', help='Number of simulation to perform', default=10, type=int)
+@click.option('--kmax', help='Maximum k-mode', default=0.5, type=float)
+@click.option('--nks', help='Number of modes', default=7, type=int)
+@click.option('--filter_kpar_min', help='Filter mode below kpar_min', default=None, type=float)
+@click.option('--filter_wedge_theta', help='Filter mode below the wedge line', default=0, type=float)
+def simu_noise_ps(uv_simu_h5, total_time_hour, out_filename, n_simu, kmax, nks, filter_kpar_min, filter_wedge_theta):
+    ''' Compute noise PS from a simulated UV coverage UV_SIMU_H5 for TOTAL_TIME_HOUR observation. '''
+
+    from ps_eor import datacube, psutil
+
+    uv_simu = TelescopGriddedUV.load(uv_simu_h5)
+    ps_gen = uv_simu.get_ps_gen(filter_kpar_min, filter_wedge_theta)
+    kbins = np.logspace(np.log10(ps_gen.kmin), np.log10(kmax), nks)
+
+    print('Simulating noise cube ...')
+    pss = {'ps': [], 'ps2d': [], 'ps3d': []}
+    pr = psutil.progress_report(n_simu)
+    for i in range(n_simu):
+        pr(i)
+        noise_cube = uv_simu.weights.simulate_noise(uv_simu.sefd, total_time_hour * 3600, hermitian=False)
+        pss['ps'].append(ps_gen.get_ps(noise_cube))
+        pss['ps2d'].append(ps_gen.get_ps2d(noise_cube))
+        pss['ps3d'].append(ps_gen.get_ps3d(kbins, noise_cube))
+
+    print('Generating power-spectra ...')
+
+    for ps_name, ps in pss.items():
+        ps_mean = ps[0].new_with_data(np.mean([k.data for k in ps], axis=0), np.mean([k.err for k in ps], axis=0))
+
+        name = f'{out_filename}_{ps_name}_{uv_simu.name}_z{uv_simu.z:.1f}_{total_time_hour}h'
+
+        print(f'Saving {ps_name} to {name} ...')
+
+        fig, ax = plt.subplots(figsize=(5, 4))
+        ps_mean.plot(ax, label='Noise')
+
+        if ps_name == 'ps3d':
+            ax.plot(ps_mean.k_mean, 2 * ps_mean.err * 1e6, label='2-sigma sensitivity')
+            ax.legend()
+
+        fig.tight_layout()
+        fig.savefig(f'{name}.pdf')
+
+        ps_mean.save_to_txt(f'{name}.txt')
+
+
 if __name__ == '__main__':
     main()
